/*
 * Orientation.java
 *
 * Created on November 27, 2005, 10:47 PM
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package javax.microedition.location;

/**
 * The Orientation class represents the physical orientation 
 * of the terminal. Orientation is described by
 * azimuth to north (the horizontal pointing direction), 
 * pitch (the vertical elevation angle) and roll (the rotation of
 * the terminal around its own longitudinal axis).
 */
public class Orientation {
    
    /**
     * Constructs a new Orientation object with the compass 
     * azimuth, pitch and roll parameters specified.</P>
     * The values are expressed in degress using floating 
     * point values.</P>
     * If the pitch or roll is undefined, the parameter 
     * shall be given as Float.NaN.
     * @param azimuth - the compass azimuth relative to true or magnetic 
     * north. Valid range: [0.0, 360.0). For
     * example, value 0.0 indicates north, 90.0 east, 
     * 180.0 south and 270.0 west.
     * @param isMagnetic a boolean stating whether the compass azimuth 
     * is given as relative to the magnetic
     * field of the Earth (=true) or to true 
     * north and gravity (=false)
     * @param pitch the pitch of the terminal in degrees. 
     * Valid range: [-90.0, 90.0]
     * @param roll the roll of the terminal in degrees. 
     * Valid range: [-180.0, 180.0)
     */
    public Orientation(float azimuth, boolean isMagnetic, float pitch, float roll) {
        if (azimuth < 0.0F || azimuth >= 360.0F) {
            throw new IllegalArgumentException("Azimuth out of range.");
        }
        if (pitch < -90.0F || pitch > 90.0F) {
            throw new IllegalArgumentException("Pitch out of range.");
        }
        if (roll < -180.0F || roll >= 180.0F) {
            throw new IllegalArgumentException("Roll out of range");
        }
        
        this.compassAzimuth = azimuth;
        this.orientationMagnetic = isMagnetic;
        this.pitch = pitch;
        this.roll = roll;
    }
    
    /**
     * Holds value of property compassAzimuth.
     */
    private float compassAzimuth;
    
    /**
     * Getter for property compassAzimuth.
     * @return Value of property compassAzimuth.
     */
    public float getCompassAzimuth() {
        return this.compassAzimuth;
    }
    
    /**
     * Holds value of property roll.
     */
    private float roll;
    
    /**
     * Getter for property roll.
     * @return Value of property roll.
     */
    public float getRoll() {
        return this.roll;
    }
    
    /**
     * Holds value of property orientationMagnetic.
     */
    private boolean orientationMagnetic;
    
    /**
     * Getter for property orientationMagnetic.
     * @return Value of property orientationMagnetic.
     */
    public boolean isOrientationMagnetic() {
        return this.orientationMagnetic;
    }
    
    /**
     * Holds value of property pitch.
     */
    private float pitch;
    
    /**
     * Getter for property pitch.
     * @return Value of property pitch.
     */
    public float getPitch() {
        return this.pitch;
    }
    
    /**
     * Returns the terminals current orientation.
     * @throws javax.microedition.location.LocationException if the implementation does not support orientation determination
     * SecurityException - if the calling application does not have 
     * a permission to query the orientation
     * @return returns an Orientation object containing the terminals 
     * current orientation or null if the
     * orientation cant be currently determined
     */
    public static javax.microedition.location.Orientation getOrientation()
    throws LocationException {
        throw new LocationException("Orientation not supported.");
    }
}
