/*
 * Coordinates.java
 *
 * Created on November 27, 2005, 7:28 PM
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package javax.microedition.location;

/**
 * The Coordinates class represents coordinates 
 * as latitude-longitude-altitude values. The latitude and
 * longitude values are expressed in degrees using 
 * floating point values. The degrees are in decimal values (rather
 * than minutes/seconds). The coordinates are given using the WGS84 datum.
 * This class also provides convenience methods for 
 * converting between a string coordinate representation and the
 * double representation used in this class.
 */
public class Coordinates {
    public static final int DD_MM = 1;
    public static final int DD_MM_SS = 2;
    
    /**
     * Constructs a new Coordinates object with the values 
     * specified. The latitude and longitude parameters
     * are expressed in degrees using floating point values. 
     * The degrees are in decimal values (rather than minutes/
     * seconds).
     * The coordinate values always apply to the WGS84 datum.
     * The Float.NaN value can be used for altitude to 
     * indicate that altitude is not known.
     * @param latitude the latitude of the location. Valid range: [-90.0, 90.0]. 
     * Positive values indicate northern latitude and negative 
     * values southern latitude.
     * @param longitude the longitude of the location. 
     * Valid range: [-180.0, 180.0). 
     * Positive values indicate eastern longitude and negative 
     * values western longitude.
     * @param altitude the altitude of the location in meters, defined as height 
     * above the WGS84 ellipsoid. Float.NaN can be used to indicate 
     * that altitude is not known.
     */
    public Coordinates(double latitude, double longitude, float altitude) {
        setLatitude(latitude);
        setLongitude(longitude);
        setAltitude(altitude);
    }

    /**
     * Holds value of property altitude.
     */
    private float altitude;

    /**
     * Getter for property altitude.
     * @return Value of property altitude.
     */
    public float getAltitude() {

        return this.altitude;
    }

    /**
     * Setter for property altitude.
     * @param altitude the altitude of the location in meters, defined as 
     * height above the WGS84 ellipsoid. 0.0 means a location at the 
     * ellipsoid surface, negative values mean the location is below the 
     * ellipsoid surface, Float.NaN that the altitude is not available.
     */
    public void setAltitude(float altitude) {
        this.altitude = altitude;
    }

    /**
     * Holds value of property latitude.
     */
    private double latitude;

    /**
     * Getter for property latitude.
     * @return Value of property latitude.
     */
    public double getLatitude() {

        return this.latitude;
    }

    /**
     * Sets the geodetic latitude for this point. Latitude is given as a 
     * double expressing the latitude in degrees in the WGS84 datum.
     * @param latitude the latitude component of this location in degrees.
     * Valid range: [-90.0, 90.0].
     */
    public void setLatitude(double latitude) {
        if (latitude < -90.0 || latitude > 90.0) {
            throw new IllegalArgumentException(
                    "Latitude must be [-90.0, 90.0].");
        }
        this.latitude = latitude;
    }

    /**
     * Holds value of property longitude.
     */
    private double longitude;

    /**
     * Getter for property longitude.
     * @return Value of property longitude.
     */
    public double getLongitude() {

        return this.longitude;
    }

    /**
     * Sets the geodetic longitude for this point. Longitude is given as a 
     * double expressing the longitude in degrees in the WGS84 datum.
     * @param longitude the longitude of the location in degrees. 
     * Valid range: [-180.0, 180.0)
     */
    public void setLongitude(double longitude) {
        if (longitude < -180.0 || longitude >= 180.0) {
            throw new IllegalArgumentException(
                    "Longitude must be [-180.0, 180.0).");
        }
        this.longitude = longitude;
    }
    
    /**
     * Calculates the geodetic distance between the two points 
     * according to the ellipsoid model of WGS84.
     * Altitude is neglected from calculations.</P>
     * The implementation shall calculate this as exactly 
     * as it can. However, it is required that the result is within 
     * 0.35% of the correct result.
     * @param to the Coordinates of the destination
     * @return the distance to the destination in meters
     */
    public float distance(javax.microedition.location.Coordinates to) {
        return Float.NaN; //Todo
    }
    
    /**
     * Converts a double representation of a coordinate 
     * with decimal degrees into a string representation.
     * There are string syntaxes supported are the same as 
     * for the convert(String) method. The implementation 
     * shall provide as many significant digits for the 
     * decimal fractions as are allowed by the string 
     * syntax definition.
     * @param coordinate a double reprentation of a coordinate
     * @param outputType identifier of the type of the string representation 
     * wanted for output The constant DD_MM_SS identifies the syntax 1 
     * and the constant DD_MM identifies the syntax 2.
     * @return a string representation of the coordinate in a representation 
     * indicated by the parameter
     */
    public static String convert(double coordinate, int outputType) {
       return "Unsupported"; //Todo
    }
    
    /**
     * Converts a String representation of a coordinate into the double 
     * representation as used in this API.
     * @param coordinate a String in either of the two representation specified 
     * @return a double value with decimal degrees that matches the 
     * string representation given as the parameter
     */
    public static double convert(String coordinate) {
       return Double.NaN; //Todo 
    }
    
    
    /**
     * Calculates the azimuth between the two points according 
     * to the ellipsoid model of WGS84. The azimuth is
     * relative to true north. The Coordinates object on which 
     * this method is called is considered the origin for the
     * calculation and the Coordinates object passed as a 
     * parameter is the destination which the azimuth is calculated
     * to. When the origin is the North pole and the destination 
     * is not the North pole, this method returns 180.0.
     * When the origin is the South pole and the destination is 
     * not the South pole, this method returns 0.0.
     * If the origin is equal to the destination, this method 
     * returns 0.0. The implementation shall calculate the result as
     * exactly as it can. However, it is required that the result is 
     * within 1 degree of the correct result.
     * @param to the Coordinates of the destination
     * @return the azimuth to the destination in degrees. Result is 
     * within the range [0.0 ,360.0).
     */
    public float azimuthTo(javax.microedition.location.Coordinates to) {
        return Float.NaN; //Todo
    }
}
