/*
 * LocationProvider.java
 *
 * Created on November 26, 2005, 12:16 PM
 */

package javax.microedition.location;

import javax.microedition.location.Coordinates;
import javax.microedition.location.Location;
import javax.microedition.location.QualifiedCoordinates;
import javax.microedition.location.Criteria;
import javax.microedition.location.LocationException;
import javax.microedition.location.LocationListener;
import javax.microedition.location.ProximityListener;

import com.motorola.iden.position.PositionConnection;
import com.motorola.iden.position.PositionDevice;
import javax.microedition.io.Connector;
import java.io.IOException;
import com.motorola.iden.position.AggregatePosition;

/**
 * This is the starting point for applications using this API and
 * represents a source of the location information. A
 * LocationProvider represents a location-providing module,
 * generating Locations.
 */
public class LocationProvider {
    public static final int AVAILABLE = 1;
    public static final int TEMPORARILY_UNAVAILABLE = 2;
    public static final int OUT_OF_SERVICE = 3;
    
    static private LocationProvider singleton;
    static private String nmeaGPGGA;
    static private String nmeaGPGLL;
    //Not implemented yet
    //static java.util.Vector proximityListeners = new java.util.Vector();
    LocationListener locationListener;
    static private javax.microedition.location.Location currentLocation;
    static private javax.microedition.location.Location previousLocation;
    static private PositionConnection motorolaConnection;
    static private AggregatePosition motPos;
    
    static private boolean stop;
    static ReadThread readThread;
    static Thread callingThread;
    
    private class ReadThread extends Thread {
        private int interval;
        private int timeout;
        private int maxAge;
        javax.microedition.location.Location location;
        
        ReadThread(int interval, int timeout, int maxAge) {
            this.interval = interval;
            this.timeout = timeout;
            this.maxAge = maxAge;
        }
        
        public void run() {
            long time;
            
            while (!stop) {
                try {
                    //Timestamp entry time of iteration
                    time = System.currentTimeMillis();
                    if (location == null
                            || location == currentLocation
                            || time - location.getTimestamp() > maxAge*1000) {
                        location = getLocation(timeout); //This can take a while
                    }
                    
                    if (location == null) {
                        continue;
                    }
                    
                    if (locationListener != null) {
                        //This should be quick, but isn't guaranteed
                        locationListener.locationUpdated(singleton, location);
                    }
                    
                    //Make time the duration we should wait until next iteration
                    time = (interval*1000) - (System.currentTimeMillis() - time);
                    time = Math.max(time, 1);
                    Thread.currentThread().sleep(time);
                } catch (InterruptedException ex) {
                } catch (javax.microedition.location.LocationException ex) {
                }
            }
        }
    }
    
    
    /** Creates a new instance of LocationProvider */
    //Todo - public?
    public LocationProvider() {
        try {
            motorolaConnection
                    = (PositionConnection)
                    Connector.open("mposition:delay=no"); //no, low, high; fix=extended cannot be used here
        } catch (IOException e) {
            System.out.println(e.getMessage());
        }
    }
    
    /**
     * Returns the current state of this LocationProvider.
     * The return value shall be one of the availability
     * status code constants defined in this class.
     * @return the availability state of this LocationProvider
     */
    public int getState() {
        if (currentLocation != null && (System.currentTimeMillis() - currentLocation.getTimestamp() < 2000)) {
            //We've had a reading in the last 2 seconds.
            return AVAILABLE;
        }
        AggregatePosition motPos = motorolaConnection.getPosition("delay=no");
        if (motPos.getLatitude() != motPos.UNAVAILABLE || motPos.getServingCellLatitude() != 0) {
            //We've got something from either GPS or tower
            return AVAILABLE;
        }
        return TEMPORARILY_UNAVAILABLE;
        //OUT_OF_SERVICE should never be returned unless radio is turned off and GPS is detached.
    }
    
    /**
     * Retrieves a Location with the constraints given by the Criteria
     * associated with this class. If no result could be retrieved,
     * a LocationException is thrown. If the location cant be
     * determined within the timeout period specified in the parameter,
     * the method shall throw a LocationException.</P>
     * If the provider is temporarily unavailable, the implementation
     * shall wait and try to obtain the location until the timeout
     * expires. If the provider is out of service, then the
     * LocationException is thrown immediately.</P>
     * Note that the individual Location returned might not fulfil
     * exactly the criteria used for selecting this LocationProvider.
     * The Criteria is used to select a location provider that
     * typically is able to meet the defined criteria, but not
     * necessarily for every individual location measurement.
     * @return a Location object
     * @param timeout a timeout value in seconds. -1 is used to indicate that
     * the implementation shall use its default timeout value for this
     * provider.
     * @throws java.lang.IllegalArgumentException if timeout < 1.
     * @throws javax.microedition.location.LocationException if the location
     * couldnt be retrieved or if the timeout period expired
     * @throws java.lang.InterruptedException if the operation is interrupted
     * by calling reset() from another thread
     */
    public synchronized javax.microedition.location.Location getLocation(
            int timeout) throws LocationException, InterruptedException,
            IllegalArgumentException {
        double lat, lon;
        float alt;
        int method;
        QualifiedCoordinates coords;
        
        if (timeout < 1) {
            throw new IllegalArgumentException("Illegal timeout.");
        }
        
        callingThread = Thread.currentThread();

// I think delay=no may be causing the repeated locations, so try not using it.
//        if (timeout < 2) {
//            motPos = motorolaConnection.getPosition("delay=no");
//        } else if (timeout <= 10) {
//            motPos = motorolaConnection.getPosition("delay=low;fix=extended");
//        } else {
//            motPos = motorolaConnection.getPosition("delay=high;fix=extended");
//        }
        
        if (timeout < 10) {
            motPos = motorolaConnection.getPosition("delay=low;fix=extended");
        } else {
            motPos = motorolaConnection.getPosition("delay=high;fix=extended");
        }        
        
        //Todo - remove
        if (motPos == null) {
            throw new NullPointerException("motPos came back null.");
        }
        
        nmeaGPGGA = motorolaConnection.getNMEASentence(PositionDevice.GPGGA);
        //nmeaGPGGA = nmeaGPGGA.substring(0, nmeaGPGGA.length()-1);
        nmeaGPGLL = motorolaConnection.getNMEASentence(PositionDevice.GPGLL);
        
        callingThread = null;
        
        previousLocation = currentLocation;
        currentLocation = new Location();
        currentLocation.setTimestamp(motPos.getTimeStamp()); //Hope this is right.
        //currentLocation.setExtraInfo(nmeaGPGGA + "\n" + nmeaGPGLL, currentLocation.mimeNMEA);
        //At least on the emulator, GPGLL can come back as null, so guard agains that.
        if (nmeaGPGLL != null) {
            nmeaGPGLL = nmeaGPGLL.substring(0, nmeaGPGLL.length()-2); //Strip CR
        }
        
        if (nmeaGPGGA != null || nmeaGPGLL != null) {
            currentLocation.setExtraInfo(nmeaGPGGA + nmeaGPGLL, currentLocation.mimeNMEA);
        }
//        currentLocation.setExtraInfo("S-Lat: " + motPos.getLatitude()
//        + " S-Lon: " + motPos.getLongitude()
//        + " T-Lat: " + motPos.getServingCellLatitude()
//        + " T-Lon: " + motPos.getServingCellLongitude(), currentLocation.mimeText);
        
//        currentLocation.setExtraInfo(MotPosSout.dump(motPos), currentLocation.mimeText);
        
        
        //What's the fix from the GPS?
        lat = uMinutesToDegrees(motPos.getLatitude());
        lon = uMinutesToDegrees(motPos.getLongitude());
        method = Location.MTE_SATELLITE;
        
        if (Double.isNaN(lat) || Double.isNaN(lon)) {
            //No fix from GPS, how about cell tower?
            lat = uMinutesToDegrees(motPos.getServingCellLatitude());
            lon = uMinutesToDegrees(motPos.getServingCellLongitude());
            method = Location.MTE_CELLID;
            //Todo - not sure what's wrong, but I can have valid numbers from
            //cell, and still have NaN shoved into location
        }
        
        if (Double.isNaN(lat) || Double.isNaN(lon)) {
            //Couldn't get a fix from the cell, either.
            currentLocation.setValid(false);
            method = 0;
        } else {
            currentLocation.setValid(true);
        }
        
        //This constructor is "broken" in that NaN fed into horz. accuracy
        //somehow becomes 0, so rather than just "leave them alone",
        //these are set again in the checks below.
        coords = new QualifiedCoordinates(lat, lon,
                Float.NaN, Float.NaN, Float.NaN);
        
//        System.out.println("coords: " + coords.getLatitude() + " "
//                + coords.getLongitude() + " "
//                + coords.getAltitude() + " "
//                + coords.getHorizontalAccuracy() + " "
//                + coords.getVerticalAccuracy());
        
        if (motPos.hasAltitude()) {
            coords.setAltitude((float)motPos.getAltitude());
        } else {
            coords.setAltitude(Float.NaN);
        }
        
        //Info from cell certainly is not within 0.0, hence the else clause.
        if (motPos.hasLatLonAccuracy() && method != Location.MTE_CELLID) {
            coords.setHorizontalAccuracy(
                    ((float)motPos.getLatLonAccuracy())/1000);
        } else {
            coords.setHorizontalAccuracy(Float.NaN);
        }
        
        if (motPos.hasAltitudeUncertainty()) {
            coords.setVerticalAccuracy(
                    ((float)motPos.getAltitudeUncertainty())/1000);
        } else {
            coords.setVerticalAccuracy(Float.NaN);
        }
        
        currentLocation.setQualifiedCoordinates(coords);
        
        //Todo - Are these NaN if I don't set them?
        if (motPos.hasSpeed()) {
            currentLocation.setSpeed(((float)motPos.getSpeed()) * .277777F); //kph -> m/s
        }
        
        if (motPos.hasTravelDirection()) {
            currentLocation.setCourse(motPos.getTravelDirection());
        }
        
        method |= Location.MTY_TERMINALBASED;
        
        if (motPos.getAssistanceUsed()) {
            method |= Location.MTA_ASSISTED;
        } else {
            method |= Location.MTA_UNASSISTED;
        }
        
        currentLocation.setLocationMethod(method);
        
        if (previousLocation != null &&
                previousLocation.getLocationMethod() != method) {
            locationListener.providerStateChanged(this, getState());
            //Proximity - monitoringStateChanged, too //Todo
        }
        
        return currentLocation;
    }
    
    private double uMinutesToDegrees(int uMinutes) {
        int deg;
        double min;
        if (uMinutes == AggregatePosition.UNAVAILABLE) {
            return Double.NaN;
        } else {
            deg = uMinutes / 6000000;
            return deg +
                    (uMinutes - (deg * 6000000)) / (100000.0 * 60.0);
        }
    }
    
    /**
     * Adds a <CODE>LocationListener</CODE> for updates at the defined interval.
     * @param locationListener the <CODE>LocationListener</CODE> to be
     * registered. If set to <CODE>null</CODE>, the registration of any
     * previously set listener is cancelled.
     * @param interval the interval in seconds. -1 is used for the default
     * interval of this provider. 0 is used to indicate that the application
     * wants to receive only provider status updates and not location updates at
     * all.
     * @param timeout timeout value in seconds, must be greater than 0. if the
     * value is -1, the default timeout forthis provider is used. Also, if the
     * interval is -1 to indicate the default, the value of this parameter has
     * no effect and the default timeout for this provider is used. If the
     * interval is 0, this parameter has no effect.
     * @param maxAge maximum age of the returned location in seconds, must be
     * greater than 0 or equal to -1 to indicate that the default maximum age
     * for this provider is used. Also, if the interval is -1 to indicate the
     * default, the value of this parameter has no effect and the default
     * maximum age for this provider is used. If the interval is 0, this
     * parameter has no effect.
     */
    public synchronized void setLocationListener(LocationListener
            locationListener, int interval, int timeout, int maxAge) {
        
        if (locationListener == null) {
            stop = true;
            locationListener = null;
            if (readThread != null) {
                readThread.interrupt();
            }
            return;
        }
        
        if (interval < -1) {
            throw new IllegalArgumentException("Illegal interval.");
        }
        
        //Todo - spec says interval != -1
        //This seems dumb, anyway, as it pretty much eliminates "as fast as
        //possible" semantics as a LocationListener
        if (interval > 0 && timeout > interval) {
            throw new IllegalArgumentException("Timeout must be < interval.");
        }
        
        if (maxAge > interval) {
            throw new IllegalArgumentException("MaxAge must be < interval.");
        }
        
        if (timeout < 1 && timeout != -1) {
            throw new IllegalArgumentException("Illegal timeout.");
        }
        
        if (maxAge < 1 && maxAge != -1) {
            throw new IllegalArgumentException("Illegal maxAge.");
        }
        
        if (interval == -1) {
            interval = 2; //Default to 2 s
            //Todo - this seems silly, but it's the spec that default interval
            //forces default maxAge
            maxAge = 10; //Default to 10 s
        }
        
        if (maxAge == -1) {
            maxAge = 10; //Default to 10 s
        }
        
        this.locationListener = locationListener;
        
        if (interval != 0) {
            stop = false;
            readThread = new ReadThread(interval, timeout, maxAge);
            readThread.start();
        }
    }
    
    /**
     * Resets the LocationProvider.</P>
     * All pending synchronous location requests will be aborted and any
     * blocked getLocation method calls will terminate with
     * <CODE>InterruptedException.</CODE></P>
     * Applications can use this method e.g. when exiting to have its threads
     * freed from blocking synchronous operations.
     */
    public void reset() {
        stop = true;
        if (callingThread != null) {
            callingThread.interrupt();
        }
        if (readThread != null) {
            readThread.interrupt();
        }
    }
    
    /**
     * This factory method is used to get an actual
     * <CODE>LocationProvider</CODE> implementation based on the defined
     *  criteria.
     * @param criteria the criteria for provider selection or <CODE>null</CODE>
     * to indicate the least restrictive criteria with default values
     * @return a LocationProvider meeting the defined criteria or null if a
     * LocationProvider that meets the defined criteria cant be returned but
     * there are other supported available or temporarily unavailable providers
     * that do not meet the criteria.
     */
    public static synchronized javax.microedition.location.LocationProvider
            getInstance(Criteria criteria) throws LocationException {
        if (criteria != null && criteria.isAddressInfoRequired()) {
            return null;
        }
        if (singleton == null) {
            singleton = new LocationProvider();
        }
        return singleton;
    }
    
    /**
     * Do not allow clone() to violate singleton design pattern.
     * @return never occurs
     * @throws java.lang.Exception will always be thrown when this method is
     * called.
     */
    public Object clone() throws Exception {
        throw new Exception("Clone not supported; LocationProvider is a singleton.");
    }
    
    /**
     * Returns the last known location that the implementation has. This is the
     * best estimate that the implementation has for the previously known
     * location.
     * @return a location object. <CODE>null</CODE> is returned if the
     * implementation doesnt have any previous location information.
     */
    public static javax.microedition.location.Location getLastKnownLocation() {
        return currentLocation;
    }
    
    /**
     *
     * @param listener
     * @param coordinates
     * @param proximityRadius
     * @throws javax.microedition.location.LocationException
     */
    public synchronized static void addProximityListener(ProximityListener listener, Coordinates
            coordinates, float proximityRadius) throws LocationException {
        throw new LocationException("Proximity monitoring not implemented for Motorola.");
    }
    
    /**
     *
     * @param listener
     */
    public synchronized static void removeProximityListener(ProximityListener listener) {
        //Do nothing; not implementd
    }
    
}
